import javafx.application.Application;
import javafx.stage.Stage;
import javafx.scene.Scene;
import javafx.scene.paint.Color;
import javafx.scene.layout.Pane;
import javafx.stage.WindowEvent;
import javafx.event.EventHandler;
import javafx.beans.value.ChangeListener;
import javafx.beans.value.ObservableValue;
import javafx.geometry.Bounds;
import javafx.application.Platform;

/**
 * アプリケーションクラス
 * <pre>
 * 当クラスからはじまります。
 * <pre>
 */
public class KApplicationSKK extends Application implements KConstant {

	/** ステージ */
	private		Stage		primaryStage	= null;

	/** ペイン */
	private		Pane		pane			= null;

	/** コントロールクラス */
	private		KCtrlSKK	ctrl			= null;

	/** パネルクラス */
	private		KPanel		panel			= null;

	/**
	 * 実行
	 * <pre>
	 * 当メソッドから、はじまります。
	 * </pre>
	 * @param args 引数
	 */
	public static void main(String[] args){

		// アプリケーションを起動します
		launch(args);

	} // end main

	/**
	 * スタート
	 * <pre>
	 * 初期化を行います。
	 * </pre>
	 */
	@SuppressWarnings("unchecked")
	@Override
	public void start(Stage primaryStage){

		// ステージを保持
		this.primaryStage = primaryStage;

		// ペインを生成
		pane = new Pane();

		// シーンを生成
		Scene scene = new Scene(pane, KConstant.WIDTH, KConstant.HEIGHT, Color.BLACK);

		// シーンを設定
		primaryStage.setScene(scene);

		// イメージクラスに自分を渡す
		KImage.setApplication(this);

		// コントロールクラス生成
		ctrl = new KCtrlSKK(this);

		// パネル取得
		panel = ctrl.getKPanel();

		// ペインにパネルを追加
		pane.getChildren().add(panel);

		// タイトル設定
		primaryStage.setTitle("SKK4 FX");

		// ウインドウサイズ変更リスナー追加
		KApplicationSKKWindowResizedListener wrl = new KApplicationSKKWindowResizedListener();
		pane.widthProperty().addListener(wrl);
		pane.heightProperty().addListener(wrl);

		// ウィンドウを閉じる際に安全に終了する
		primaryStage.setOnCloseRequest(req -> {
			// JavaFXアプリケーションを終了する
			Platform.exit();
			// 他に終了していないスレッドを終了する
			System.exit(0);
		});

		// ウィンドウ表示
		primaryStage.show();

		// コントロールクラスを開始
		ctrl.start();

	} // end KApplicationSKK

	/**
	 * ウインドウサイズ変更イベント
	 */
	private class KApplicationSKKWindowResizedListener implements ChangeListener {

		@Override
		public void changed(ObservableValue observable, Object oldValue, Object newValue) {

			// ペイン（ウインドウ）のサイズを取得
			double widthPane = KApplicationSKK.this.pane.getWidth();
			double heightPane = KApplicationSKK.this.pane.getHeight();

			// 比率を求める
			double ratioWidth = (float)widthPane / KConstant.WIDTH;
			double ratioHeight = (float)heightPane / KConstant.HEIGHT;

			// 比率の小さい方を求める
			double ratio = (ratioWidth < ratioHeight)?ratioWidth: ratioHeight;

			// パネルを拡大縮小する
			panel.setScaleX(ratio);
			panel.setScaleY(ratio);

			// ペイン上のパネル境界を取得する
			Bounds bounds = panel.getBoundsInParent();

			// ペイン上のパネルの幅・高さを取得する
			double widthPanel = bounds.getWidth();
			double heightPanel = bounds.getHeight();

			// ペイン上のパネルを再配置する
			// 式の前半は、ペインとパネルの差分を移動
			// 式の後半は、拡大／縮小した分の差分を移動している
			panel.setLayoutX((widthPane - widthPanel) / 2 + ((heightPanel - panel.getWidth()) / 2));
			panel.setLayoutY((heightPane - heightPanel) / 2 + ((heightPanel - panel.getHeight()) / 2));

		} // end changed

	} // end KApplicationSKKWindowResizedListener

}
