import javafx.application.Application;

/**
 * コントロールクラス
 * <pre>
 * 全体的な制御を行うスーパークラス。
 * 実際には、継承したクラスで制御を行ってください。
 * </pre>
 */
abstract public class KCtrl {

	/** アプリケーション */
	private		Application			application			= null;

	/**
	 * コンストラクタ
	 * @param application アプリケーション
	 */
	public KCtrl(Application application){

		this.application = application;

	} // end KCtrl

	/**
	 * 開始
	 * <pre>
	 * 当オブジェクトの処理を開始します。
	 * 継承先で定義をしてください。
	 * </pre>
	 */
	abstract public void start();

	/**
	 * 実行
	 * <pre>
	 * 当オブジェクトのふるまいを決定します。
	 * 継承先で定義をしてください。
	 * </pre>
	 * @return  true：正常 ／ false：一時停止
	 */
	abstract public boolean run();

	/**
	 * アプリケーション取得
	 * <pre>
	 * 当クラス自体はApplicationクラスを継承はしていないが、Applicationより生成される作りとしている。
	 * そのため、自分を生成したクラスを内部的に保持しておき、その後の処理に適宜利用する。
	 * </pre>
	 * @return Application アプリケーション
	 */
	public Application getApplication(){

		return application;

	} // getApplication

}
