import java.util.*;
import javafx.scene.image.Image;
import javafx.scene.Node;
import javafx.scene.input.KeyCode;

/**
 * 自機
 */
public class KSpriteShip extends KSprite implements KConstant {

	/** 移動量 */
	private		double				amount				= 0.0d;

	/** ビーム発射間隔 */
	private		int					intervalBeam		= 20;

	/** ビームスピード */
	private		int					speedBeam			= 12;

	/** ビーム時間 */
	private		int					timeBeam			= intervalBeam;

	/** 被弾時間 */
	private		int					timeHidan			= 0;

	/** 被弾立て直し時間 */
	private final int				HIDAN				= 200;

	/** Zキー押下フラグ */
	private		boolean				flgZ				= false;

	/** Aキー押下フラグ */
	private		boolean				flgA				= false;

	/** Beamオートフラグ */
	private		boolean				flgAutoBeam			= true;

	/** 爆発フラグ */
	private		boolean				flgBomb				= false;

	/** 爆発時間 */
	private		int					timeBomb			= -1;

	/**
	 * イメージ初期化
	 * <pre>
	 * イメージの取り込みを行います。
	 * </pre>
	 */
	public static void initImage() {

		Image imageShip = null;
		Image imageShip_R = null;
		Image imageShip_L = null;
		Image imageB01 = null;
		Image imageB02 = null;

		imageShip = KImage.getImage("image\\imageShip.gif");
		imageShip_R = KImage.getImage("image\\imageShip_R.gif");
		imageShip_L = KImage.getImage("image\\imageShip_L.gif");
		imageB01 = KImage.getImage("image\\imageBomb1.gif");
		imageB02 = KImage.getImage("image\\imageBomb2.gif");

		KSpriteShip.setImage(KSpriteShip.class, new Image[]{
			imageShip,
			imageShip_R,
			imageShip_L,
			imageB01,
			imageB02
		});

	} // end initImage

	/**
	 * コンストラクタ
	 */
	public KSpriteShip(KPanel panel){

		super(panel);

	} // end KSpriteShip

	/**
	 * 初期化
	 */
	public void init(){

		// 位置初期化
		this.setLayoutX(284);
		this.setLayoutY(480);

		// ビーム発射間隔
		intervalBeam		= 20;
		// ビームスピード
		speedBeam			= 12;
		// ビーム時間.
		timeBeam			= intervalBeam;
		// 被弾時間.
		timeHidan			= 0;

		// 爆発フラグ
		flgBomb = false;

		// イメージ初期化
		setImage(KSpriteShip.class, 0);

	} // end init

	/**
	 * 実行.
	 * <pre>
	 * 当オブジェクトのふるまいを決定します。
	 * 継承先で定義をしてください。
	 * <pre>
	 */
	public void run(){

		// 撃墜された場合、
		if(flgBomb){

			timeBomb++;

			if(timeBomb == 10){
				// 爆発
				setImage(KSpriteShip.class, 4);
			}else if(timeBomb == 30){
				// 爆発
				setImage(KSpriteShip.class, 3);
			}else if(timeBomb == 45){
				// スプライトを消す
				super.setVisible(false);
			}

			// リターン
			return;

		} // end if 撃墜された場合

		//
		// 移動
		//

		// ステージクリア中でもゲームオーバー中でもない場合、
		if(((KCtrlSKK)panel.getKCtrl()).getPhase() != KCtrlSKK.PHASE_STAGE_CLEAR &&
				((KCtrlSKK)panel.getKCtrl()).getPhase() != KCtrlSKK.PHASE_END){

			double x = super.getLayoutX();
			double y = super.getLayoutY();

			double mx = 0;
			double my = 0;

			// ←
			if(panel.isKeyPressed(KeyCode.LEFT)){
				if(x > 0){
					mx = mx - (4.0 + amount);
				}
			}

			// →
			if(panel.isKeyPressed(KeyCode.RIGHT)){
				if(x < (WIDTH - this.getWidth())){
					mx = mx + (4.0 + amount);
				}
			}

			// ↑
			if(panel.isKeyPressed(KeyCode.UP)){
				if(y > 0){
					my = my - (4.0 + amount);
				}
			}

			// ↓
			if(panel.isKeyPressed(KeyCode.DOWN)){
				if(y < (HEIGHT - this.getHeight())){
					my = my + (4.0 + amount);
				}
			}

			// 機体方向−まっすぐ
			if(mx == 0){
				setImage(KSpriteShip.class, 0);
			// 右
			}else if(mx > 0){
				setImage(KSpriteShip.class, 1);
			// 左
			}else{
				setImage(KSpriteShip.class, 2);
			}

			x = x + mx;
			y = y + my;

			super.setLayoutX(x);
			super.setLayoutY(y);

			//
			// ビーム
			//
			this.timeBeam++;

			// ビームオート かつ 時間が来たら、
			// もしくは、キーを押していなかった時に押した時、
			if(this.flgAutoBeam && this.timeBeam >= this.intervalBeam ||
							panel.isKeyPressed(KeyCode.Z) && !flgZ){

				this.timeBeam = 0;

				// ビームを生成、発射
				KSpriteBeam1 spriteBeam1 = new KSpriteBeam1(panel, speedBeam);
				panel.addSprite(LAYOUT_TEKI, spriteBeam1);
				spriteBeam1.setLayoutX(x);
				spriteBeam1.setLayoutY(y);

				// ビーム音スタート
				((KCtrlSKK)panel.getKCtrl()).startWaveBeam();

			}

			// Aキーを押していなかった時に押した時、
			if(panel.isKeyPressed(KeyCode.A) && !flgA){
				// ビームオート制御
				this.flgAutoBeam = !this.flgAutoBeam;
			}

			// Zキー制御
			flgZ = panel.isKeyPressed(KeyCode.Z);

			// Aキー制御
			flgA = panel.isKeyPressed(KeyCode.A);

		} // end if ステージクリア中ではない場合、

		//
		// 判定
		//

		// スプライト一覧取得
		List<Node> spriteLst = panel.getSpriteLst(LAYOUT_TEKI);

		// スプライト一覧の件数を取得
		int countSpriteLst = spriteLst.size();

		// スプライト一覧の件数分、ループ
		for(int i = 0; i < spriteLst.size(); i++){

			// スプライトを取得
			KSprite sprite = (KSprite)spriteLst.get(i);

			// nullの場合、
			if(sprite == null){
				// コンティニュー
				continue;
			}

			// XY取得
			double x = super.getLayoutX();
			double y = super.getLayoutY();

			// 敵XY取得
			double xSprite = sprite.getLayoutX();
			double ySprite = sprite.getLayoutY();

			// 大きさ取得
			double intWidthSprite = sprite.getWidth();
			double intHeightSprite = sprite.getHeight();

			// 当たり判定
			if(x + 4 < (xSprite + intWidthSprite) && xSprite < (x + this.getWidth() - 4) &&
							y + 4 < (ySprite + intHeightSprite) && ySprite < (y + this.getHeight() - 4) ){

			// 当たっていない場合、
			}else{
				// コンティニュー
				continue;
			}

			// 地上物の場合、
			if(sprite instanceof KSpriteG00){

				// 何もしない

			// 敵の場合、
			}else if(sprite instanceof KSpriteT00){

				// 被弾中の場合、
				if(timeHidan > 0){
					// コンティニュー
					continue;
				}

				// 敵にキャスト
				KSpriteT00 spriteT00 = (KSpriteT00)sprite;

				// 当たり判定しない場合、
				if(!spriteT00.isHanteiAtari()){
					// コンティニュー
					continue;
				}

				// 爆発している場合、
				if(spriteT00.isBomb()){
					// コンティニュー
					continue;
				}

				// 敵を爆発させる
				spriteT00.bomb();

				// 被弾処理
				hidan();

			// ビームの場合、
			}else if(sprite instanceof KSpriteT99){

				// 被弾中の場合、
				if(timeHidan > 0){
					// コンティニュー
					continue;
				}

				// ビームを消去する
				sprite.removeSprite();

				// 被弾処理
				hidan();

			// Sの場合、
			}else if(sprite instanceof KSpriteS01){

				// スプライトを削除
				sprite.removeSprite();

				// 移動量が最大以外の場合、
				if(amount <= 2){

					// 移動量を増やす
					amount = amount + 0.5f;

				}

				// アイテム取得スタート
				((KCtrlSKK)panel.getKCtrl()).startWaveItemGet();

			// Mの場合、
			}else if(sprite instanceof KSpriteS02){

				// スプライトを削除
				sprite.removeSprite();

				// ビーム発射間隔が最大以外の場合、
				if(intervalBeam > 5){

					// ビーム発射間隔を短くする
					intervalBeam = intervalBeam - 3;

				}

				// アイテム取得スタート
				((KCtrlSKK)panel.getKCtrl()).startWaveItemGet();

			// Hの場合、
			}else if(sprite instanceof KSpriteS03){

				// スプライトを削除
				sprite.removeSprite();

				// ビームスピードが最大以外の場合、
				if(speedBeam < 20){

					// ビームスピードを早くする
					speedBeam = speedBeam + 4;

				}

				// アイテム取得スタート
				((KCtrlSKK)panel.getKCtrl()).startWaveItemGet();

			// 1の場合、
			}else if(sprite instanceof KSpriteS04){

				// スプライトを削除
				sprite.removeSprite();

				// 防御加算
				((KCtrlSKK)panel.getKCtrl()).addShield(1);

				// アイテム取得スタート
				((KCtrlSKK)panel.getKCtrl()).startWaveItemGet();

			}

		} // end for スプライト一覧の件数分、ループ

		// 被弾時間計算
		timeHidan = (timeHidan > 0)?--timeHidan: 0;

		// 自機を表示する
		super.setVisible(true);

		// 被弾中の場合、
		if(timeHidan > 0){

			// 表示しない場合、
			if((timeHidan % 20) > 10){
				// 自機を表示しない
				super.setVisible(false);
			}

		}

	} // end run

	/**
	 * 被弾処理
	 */
	public void hidan(){

		// 防御減算
		int shield = ((KCtrlSKK)panel.getKCtrl()).addShield(-1);

		// 防御が無くなった場合、
		if(shield == 0){

			// 撃墜
			flgBomb = true;

			// 爆発音スタート
			((KCtrlSKK)panel.getKCtrl()).startWaveBomb();

			setImage(KSpriteShip.class, 3);

			// リターン
			return;

		// 防御が残っている場合、
		}else{

			//  敵接触音スタート
			((KCtrlSKK)panel.getKCtrl()).startWaveCollision();

		} // end if

		// 被弾時間
		timeHidan = HIDAN;

	} // end hidan

	/**
	 * 撃墜判定
	 * <pre>
	 * 撃墜したか判定します。
	 * <pre>
	 * @return false−撃墜していない ／ true−撃墜
	 */
	public boolean isBomb(){

		// リターン
		return flgBomb;

	} // end isBomb

	/**
	 * 被弾判定
	 * <pre>
	 * 被弾中か判定します。
	 * <pre>
	 * @return false−被弾中ではない ／ true−被弾中
	 */
	public boolean isHidan(){

		// リターン
		return timeHidan > 0;

	} // end isHidan

	/**
	 * Beamオートフラグ取得
	 * <pre>
	 * Beamオートフラグを取得します。
	 * <pre>
	 * @return boolean true−オート
	 */
	public boolean isAutoBeam(){

		return this.flgAutoBeam;

	}

	/**
	 * Beamオートフラグ設定
	 * <pre>
	 * Beamオートフラグを設定します。
	 * <pre>
	 * @param flgAutoBeam true−オート
	 */
	public void setAutoBeam(boolean flgAutoBeam){

		this.flgAutoBeam = flgAutoBeam;

	}

}
