import java.util.*;
import javafx.scene.image.Image;
import javafx.scene.Node;
import javafx.scene.input.KeyCode;

/**
 * 自機２号機
 */
public class KSpriteShip2 extends KSprite implements KConstant {

	/** フェーズ */
	private		int					phase				= 0;

	/** ビーム発射間隔 */
	private		int					intervalBeam		= 30;

	/** ビーム時間 */
	private		int					timeBeam			= intervalBeam;

	/** 爆発時間 */
	private		int					timeBomb			= -1;

	/** Zキー押下フラグ */
	private		boolean				flgZ				= false;

	/**
	 * イメージ初期化
	 * <pre>
	 * イメージの取り込みを行います。
	 * </pre>
	 */
	public static void initImage() {

		Image imageShip2 = null;
		Image imageShip2_R = null;
		Image imageShip2_L = null;
		Image imageB01 = null;
		Image imageB02 = null;

		imageShip2 = KImage.getImage("image\\imageShip2.gif");
		imageShip2_R = KImage.getImage("image\\imageShip2_R.gif");
		imageShip2_L = KImage.getImage("image\\imageShip2_L.gif");
		imageB01 = KImage.getImage("image\\imageBomb1.gif");
		imageB02 = KImage.getImage("image\\imageBomb2.gif");

		KSpriteShip2.setImage(KSpriteShip2.class, new Image[]{
			imageShip2,
			imageShip2_R,
			imageShip2_L,
			imageB01,
			imageB02
		});

	} // end initImage

	/**
	 * コンストラクタ
	 */
	public KSpriteShip2(KPanel panel){

		super(panel);

		// 位置初期化
		this.setLayoutX(284);
		this.setLayoutY(-getHeight());

	} // end KSpriteShip2

	/**
	 * 初期化
	 */
	public void init(){

		// 位置初期化
		this.setLayoutX(284);
		this.setLayoutY(-getHeight());

		// ビーム発射間隔
		intervalBeam		= 30;
		// ビーム時間.
		timeBeam			= intervalBeam;

		// イメージ初期化
		setImage(KSpriteShip2.class, 0);

	} // end init

	/**
	 * 実行.
	 * <pre>
	 * 当オブジェクトのふるまいを決定します。
	 * 継承先で定義をしてください。
	 * <pre>
	 */
	public void run(){

		// 自機を取得
		KSpriteShip spriteShip = ((KCtrlSKK)panel.getKCtrl()).getSpriteShip();

		// 座標を取得
		double xSpriteShip = spriteShip.getLayoutX();
		double ySpriteShip = spriteShip.getLayoutY();

		// 座標を取得
		double x = super.getLayoutX();
		double y = super.getLayoutY();

		//
		// 座標を決定
		//

		// フェーズ＝０の場合、
		if(phase == 0){

			// 相対的な位置を取得
			double xx = xSpriteShip - x;
			double yy = ySpriteShip - y;

			// 移動方向を決定
			if(xx < -2){
				xx = -2;
			}else if(xx > 2){
				xx = 2;
			}else{
				xx = 0;
			}

			if(yy < -2){
				yy = -2;
			}else if(yy > 2){
				yy = 2;
			}else{
				yy = 0;
			}

			// 座標を修正
			x = x + xx;
			y = y + yy;

			// 自機と近づいた場合、
			if(Math.abs(xSpriteShip - x) < 4 && Math.abs(ySpriteShip - y) < 4){

				// フェーズを変更
				phase = 1;

				// アイテム取得スタート
				((KCtrlSKK)panel.getKCtrl()).startWaveItemGet();

			} // end if 自機と近づいた場合

		// フェーズ＝１の場合、
		}else if(phase == 1){

			// 座標を修正
			x = xSpriteShip;
			y = ySpriteShip;

		}

		// 座標を設定
		super.setLayoutX(x);
		super.setLayoutY(y);

		//
		// ビーム
		//

		// フェーズ＝１の場合、
		if(phase == 1){

			this.timeBeam++;

			// ビームオート かつ 時間が来たら、
			// もしくは、Zキーを押していなかった時に押した時、
			if(spriteShip.isAutoBeam() && this.timeBeam >= this.intervalBeam ||
				panel.isKeyPressed(KeyCode.Z) && !flgZ){

				this.timeBeam = 0;

				// 左ビームを生成、発射
				KSpriteBeam2 spriteBeamL = new KSpriteBeam2(panel, -8, 8);
				panel.addSprite(LAYOUT_TEKI, spriteBeamL);
				spriteBeamL.setLayoutX(x);
				spriteBeamL.setLayoutY(y + 20);

				// 右ビームを生成、発射
				KSpriteBeam2 spriteBeamR = new KSpriteBeam2(panel, 8, 8);
				panel.addSprite(LAYOUT_TEKI, spriteBeamR);
				spriteBeamR.setLayoutX(x + 20);
				spriteBeamR.setLayoutY(y + 20);

			}

			// Zキー制御
			flgZ = panel.isKeyPressed(KeyCode.Z);

		} // end if フェーズ＝１の場合

		// 自機２を表示する
		super.setVisible(spriteShip.isVisible());

/*

		// 撃墜された場合、
		if(flgBomb){

			timeBomb++;

			if(timeBomb == 10){
				// 爆発
				setImage(KSpriteShip2.class, 4);
			}else if(timeBomb == 30){
				// 爆発
				setImage(KSpriteShip2.class, 3);
			}else if(timeBomb == 45){
				// スプライトを消す
				super.setVisible(false);
			}

			// リターン
			return;

		} // end if 撃墜された場合

		//
		// 移動
		//

		// ステージクリア中でもゲームオーバー中でもない場合、
		if(((KCtrlSKK)panel.getKCtrl()).getPhase() != KCtrlSKK.PHASE_STAGE_CLEAR &&
				((KCtrlSKK)panel.getKCtrl()).getPhase() != KCtrlSKK.PHASE_END){

			double x = super.getLayoutX();
			double y = super.getLayoutY();

			double mx = 0;
			double my = 0;

			// 機体方向−まっすぐ
			if(mx == 0){
				setImage(KSpriteShip2.class, 0);
			// 右
			}else if(mx > 0){
				setImage(KSpriteShip2.class, 1);
			// 左
			}else{
				setImage(KSpriteShip2.class, 2);
			}

			x = x + mx;
			y = y + my;

			super.setLayoutX(x);
			super.setLayoutY(y);

			// Zキー制御
			flgZ = panel.isKeyPressed(KeyCode.Z);

			// Aキー制御
			flgA = panel.isKeyPressed(KeyCode.A);

		} // end if ステージクリア中ではない場合、

		// XY取得
		double x = super.getLayoutX();
		double y = super.getLayoutY();

*/



	} // end run

}
